package com.fundi.framework.common.displaylist;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.io.Serializable;
import java.util.ArrayList;

import javax.xml.bind.annotation.XmlRootElement;

@XmlRootElement
public class ListFilter implements Comparable<ListFilter>, Serializable {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private String description = ""; //$NON-NLS-1$
	private ArrayList<ListFilterCondition> conditions = new ArrayList<ListFilterCondition>();
	private boolean active = false;
	private boolean summary = false;
	
	public ListFilter() {
		
		super();
	}
	
	public ListFilter(String description) {
		
		super();
		
		this.description = description;
	}
	
	public String[] getFieldLabels(DisplayList list) {
		
		int idx;
		ListFieldDefn tempDefn;
		ArrayList<ListFieldDefn> defns = (ArrayList<ListFieldDefn>) list.getFieldDefns().clone();
		String[] labels = new String[defns.size()];
		
		//Firstly sort the definitions according to their label.
		for (idx = 1; idx < defns.size(); idx++) {
			if (defns.get(idx).getLabel().compareToIgnoreCase(defns.get(idx - 1).getLabel()) < 0) {
				tempDefn = defns.get(idx);
				defns.set(idx, defns.get(idx - 1));
				defns.set(idx - 1, tempDefn);
				idx = 0;
			}
		}
		
		for (idx = 0; idx < labels.length; idx++) {
			labels[idx] = defns.get(idx).getLabel();
		}

		return labels;
	}
	
	public ArrayList<ListFilterCondition> getConditions() {
		
		return conditions;
	}
	
	public void clearConditions() {
		
		conditions.clear();
	}
	
	public void addCondition(ListFieldDefn defn, String operator, String uneditedValue, boolean highlight, int red, int green, int blue,
											boolean caseSensitive) {
		
		conditions.add(new ListFilterCondition(defn, operator, uneditedValue, highlight, red, green, blue, caseSensitive));
	}
	
	public boolean fieldPassesFilter(ListField field) {
		
		return fieldPassesFilter(field, false);
	}
	
	public boolean rowPassesFilter(ArrayList<String> fields, ListRow row) {
		
		boolean passes = true;
		int idx;
		ListFilterCondition condition;
		int idxFld;
		boolean anyFieldContains;
		
		//Conditions are ANDed.
		for (idx = 0; idx < conditions.size() && passes; idx++) {
			condition = conditions.get(idx);
			if (!condition.isHighlight()) {
				if (condition.getFieldDefn().getLabel().equals(ListFilterCondition.anyField)) {
					anyFieldContains = false;
					for (idxFld = 0; idxFld < fields.size() && !anyFieldContains; idxFld++) {
						if (row.getField(fields.get(idxFld)) != null) {  //Can happen with CSL API rows.
							anyFieldContains = condition.valueMeetsCondition(row.getField(fields.get(idxFld)), false);
						}
					}
					if (!anyFieldContains) {  //No field in the row contains the value;
						passes = false;
					}
				}
				else {
					for (idxFld = 0; idxFld < fields.size() && passes; idxFld++) {
						if (row.getField(fields.get(idxFld)) != null) {  //Can happen with CSL API rows.
							passes = condition.valueMeetsCondition(row.getField(fields.get(idxFld)), false);
						}
						else {
							//If the missing field is the one in the condition, then it can't pass the condition.
							if (fields.get(idxFld).equals(condition.getFieldDefn().getLabel())) {
								passes = false;
							}
						}
					}
				}
			}
		}
		
		return passes;
	}
	
	public boolean fieldPassesFilter(ListField field, boolean forceNumericTest) {
		
		int idx;
		boolean passes = true;
		ListFilterCondition condition;
		
		//Conditions are ANDed.
		for (idx = 0; idx < conditions.size() && passes; idx++) {
			condition = conditions.get(idx);
			if (!(condition.isHighlight())) {
				if (!(condition.valueMeetsCondition(field, forceNumericTest))) {
					passes = false;
				}
			}
		}
		
		return passes;
	}
	
	public ListFieldColor getFieldHighlight(ListField field)  {
		
		return getFieldHighlight(field, false);
	}

	public ListFieldColor getFieldHighlight(ListField field, boolean forceNumericTest) {
		
		ListFieldColor color = new ListFieldColor();
		int idx;
		boolean passes = false;
		ListFilterCondition condition;
		
		if (field != null) {
			for (idx = 0; idx < conditions.size() && !passes; idx++) {
				condition = conditions.get(idx);
				if (condition.isHighlight()) {
					if (condition.valueMeetsCondition(field, forceNumericTest)) {
						passes = true;
						color.setRed(condition.getRed());
						color.setGreen(condition.getGreen());
						color.setBlue(condition.getBlue());
					}
				}
			}
		}
		
		return color;
	}
	
	public ArrayList<String> getHighlightFieldsInSequence() {
		
		ArrayList<String> labels = new ArrayList<String>();
		int idx;
		ListFilterCondition condition;
		
		for (idx = 0; idx < conditions.size(); idx++) {
			condition = conditions.get(idx);
			if (condition.isHighlight()) {
				labels.add(condition.getFieldDefn().getLabel());
			}
		}
		
		return labels;
	}
	
	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public boolean isActive() {
		return active;
	}

	public void setActive(boolean active) {
		this.active = active;
	}

	public void setConditions(ArrayList<ListFilterCondition> conditions) {
		this.conditions = conditions;
	}

	public boolean isSummary() {
		return summary;
	}

	public void setSummary(boolean summary) {
		this.summary = summary;
	}

	@Override
	public int compareTo(ListFilter arg0) {
		
		return description.toLowerCase().compareTo(arg0.getDescription().toLowerCase());
	}
}
