package com.fundi.framework.common.navigation;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.fundi.framework.common.util.Collator;
import com.fundi.framework.common.util.StringEditor;

@SuppressWarnings("rawtypes")
public class MenuItem implements Comparable {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private String caption = ""; //$NON-NLS-1$
	private String category = ""; //$NON-NLS-1$
	private String tooltipText = ""; //$NON-NLS-1$
	private String imageFilePath = null;
	private String imagePluginId = null;
	private Object provider = null;
	private Object data = null;
	private	Map<String, Object> dataMap = new HashMap<String, Object>();
	private ArrayList<MenuItem> subItems = new ArrayList<MenuItem>();
	private String compareKey = ""; //$NON-NLS-1$
	private boolean placeOnToolbar = false;
	private Style style = null;
	
	public enum Style {
		CHECK,
		CASCADE,
		PUSH,
		RADIO,
		SEPARATOR
	}
	public MenuItem() {
		
		super();
	}

	public MenuItem(String caption, String category, String tooltipText, Object provider) {
		
		super();
		
		initialise(caption, category, tooltipText, provider, null, null);
	}
	
	public MenuItem(String caption, String category, String tooltipText, Object provider, Object data) {
		
		super();
		
		initialise(caption, category, tooltipText, provider, data, null);
	}
	
	public MenuItem(String caption, String category, String tooltipText, Object provider, Object data, Style style) {
		
		super();
		
		initialise(caption, category, tooltipText, provider, data, style);
	}
	
	private void initialise(String caption, String category, String tooltipText, Object provider, Object data, Style style) {
		
		this.caption = caption;
		this.category = category;
		this.tooltipText = tooltipText;
		this.provider = provider;
		this.data = data;
		this.style = style;
		
		if (category == null) {
			category = ""; //$NON-NLS-1$
		}
		
		compareKey = (new StringEditor(category)).pad(50) + (new StringEditor(caption)).pad(50); 
	}

	public String getCaption() {
		
		return caption;
	}

	public void setCaption(String caption) {
		
		this.caption = caption;
	}
	
	public String getTooltipText() {
		
		return tooltipText;
	}
	
	public void setTooltipText(String tooltipText) {
		
		this.tooltipText = tooltipText;
	}

	public String getCategory() {
		
		return category;
	}

	public void setCategory(String category) {
		
		this.category = category;
		if (category == null) {
			category = ""; //$NON-NLS-1$
		}
	}
	
	public void addSubItem(MenuItem subItem) {
		
		subItems.add(subItem);
	}
	
	public void addSubItems(ArrayList<MenuItem> items) {
		
		Iterator<MenuItem> itItm;
		
		for (itItm = items.iterator(); itItm.hasNext();) {
			subItems.add(itItm.next());
		}
	}
	    
	public ArrayList<MenuItem> getSubItems() {
		
		return subItems;
	}
	
	public void setSubItems(ArrayList<MenuItem> items) {
		
		subItems.clear();
		subItems.addAll(items);
	}
	
	public boolean hasSubItems() {
		
		return subItems.size() > 0;
	}
	
	public boolean isSeparator() {
		
		if (caption == null) {
			return true;
		}
		else {
			return caption.length() == 0;
		}
	}
	
	public String getCompareKey() {
		
		return compareKey;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((caption == null) ? 0 : caption.hashCode());
		result = prime * result
				+ ((provider == null) ? 0 : provider.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		MenuItem other = (MenuItem) obj;
		if (caption == null) {
			if (other.caption != null)
				return false;
		} else if (!caption.equals(other.caption))
			return false;
		/*if (provider == null) {
			if (other.provider != null)
				return false;
		} else if (!provider.equals(other.provider))
			return false;*/
		return true;
	}

	@Override
	public int compareTo(Object arg0) {
		
		return (new Collator()).compare(getCompareKey(), ((MenuItem) arg0).getCompareKey());
	}

	public Object getProvider() {
		
		return provider;
	}

	public Object getData() {
		
		return data;
	}
	
	public Object getData(String key) {
		
		return dataMap.get(key);
	}

	public void setData(Object data) {
		
		this.data = data;
	}
	
	public void setData(String key, Object data) {
		
		dataMap.put(key, data);
	}

	public String getImageFilePath() {
		
		return imageFilePath;
	}

	public void setImageFilePath(String imageFilePath) {
		
		this.imageFilePath = imageFilePath;
	}

	public String getImagePluginId() {
		
		return imagePluginId;
	}

	public void setImagePluginId(String imagePluginId) {
		
		this.imagePluginId = imagePluginId;
	}

	public boolean placeOnToolbar() {
		
		return placeOnToolbar;
	}

	public void setPlaceOnToolbar(boolean placeOnToolbar) {
		
		this.placeOnToolbar = placeOnToolbar;
	}

	public Style getStyle() {
		return style;
	}

	public void setStyle(Style style) {
		this.style = style;
	}
}
