/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

package com.fundi.framework.common.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

/**
 * A Wrapper to any List<E> that provides the convenience methods .first() and .last()
 * 
 * Essentially this just supports a slightly terser coding style when building lists, eg. the following code from DialogControlManager:
 * 
 *	protected ListXWrapper<Composite>	groups;
 *
 * 	public Composite createComposite(Composite parent, int numColumns) {
 *		GridLayout	gridLayout;
 *		GridData	gridData;
 *		
 *		groups.add(new Composite(parent, SWT.NONE));
 *		groups.last().setFont(frameFont);
 *		gridLayout = new GridLayout(numColumns, false);		
 *		groups.last().setLayout(gridLayout);
 *		gridData = new GridData(SWT.FILL, SWT.TOP, true, false);
 *		groups.last().setLayoutData(gridData);
 *		
 *		return groups.last();
 *	}
 * 
 */
public class ListXWrapper<E> implements List<E> {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	List<E> list;
	
	public ListXWrapper() {
		list = new ArrayList<E>();
	}
	
	public ListXWrapper(int initialCapacity) {
		list = new ArrayList<E>(initialCapacity);
	}

	public ListXWrapper(List<E> list) {
		this.list = list;
	}
	
	public ListXWrapper(Collection<? extends E> c) {
		list = new ArrayList<E>(c);
	}
	
	public int size() {
		return list.size();
	}

	public boolean isEmpty() {
		return list.isEmpty();
	}

	public boolean contains(Object o) {
		return list.contains(o);
	}

	public Iterator<E> iterator() {
		return list.iterator();
	}

	public Object[] toArray() {
		return list.toArray();
	}

	public <T> T[] toArray(T[] a) {
		return list.toArray(a);
	}

	public boolean add(E e) {
		return list.add(e);
	}

	public boolean remove(Object o) {
		return list.remove(o);
	}

	public boolean containsAll(Collection<?> c) {
		return list.containsAll(c);
	}

	public boolean addAll(Collection<? extends E> c) {
		return list.addAll(c);
	}

	public boolean addAll(int index, Collection<? extends E> c) {
		return list.addAll(index, c);
	}

	public boolean removeAll(Collection<?> c) {
		return list.removeAll(c);
	}

	public boolean retainAll(Collection<?> c) {
		return list.retainAll(c);
	}

	public void clear() {
		list.clear();
	}

	public boolean equals(Object o) {
		return list.equals(o);
	}

	public int hashCode() {
		return list.hashCode();
	}

	public E get(int index) {
		return list.get(index);
	}

	public E set(int index, E element) {
		return list.set(index, element);
	}

	public void add(int index, E element) {
		list.add(index, element);
	}

	public E remove(int index) {
		return list.remove(index);
	}

	public int indexOf(Object o) {
		return list.indexOf(o);
	}

	public int lastIndexOf(Object o) {
		return list.lastIndexOf(o);
	}

	public ListIterator<E> listIterator() {
		return list.listIterator();
	}

	public ListIterator<E> listIterator(int index) {
		return list.listIterator(index);
	}

	public List<E> subList(int fromIndex, int toIndex) {
		return list.subList(fromIndex, toIndex);
	}

	//Added convenience methods
	
	public E first() {
		return (list.isEmpty() ? null : list.get(0));
	}
	
	public E last() {
		return (list.isEmpty() ? null : list.get(list.size() - 1));
	}
}
