/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

package com.fundi.framework.common.util;

import java.io.File;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;

public class SharedFile extends File {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private static final long serialVersionUID = 1L;

	private List<Object> users = new ArrayList<Object>();

	public SharedFile(String pathname) {
		super(pathname);
	}

	public SharedFile(URI uri) {
		super(uri);
	}

	public SharedFile(String parent, String child) {
		super(parent, child);
	}

	public SharedFile(File parent, String child) {
		super(parent, child);
	}

	public SharedFile(String pathname, Object userProcess) {
		super(pathname);
		registerUser(userProcess);
	}

	public SharedFile(URI uri, Object userProcess) {
		super(uri);
		registerUser(userProcess);
	}

	public SharedFile(String parent, String child, Object userProcess) {
		super(parent, child);
		registerUser(userProcess);
	}

	public SharedFile(File parent, String child, Object userProcess) {
		super(parent, child);
		registerUser(userProcess);
	}

	public synchronized void registerUser(Object userProcess) {
		if (!users.contains(userProcess)) {
			users.add(userProcess);
		}
	}

	@Override
	public boolean delete() {
		deleteOnExit();	//an insurance policy
		if (users.isEmpty()) {
			return super.delete();
		}
		
		return false;	//because until the caller is removed (if registered) then this file won't get deleted (except for on jvm exit)
	}
	
	public synchronized boolean delete(Object userProcess) {
		users.remove(userProcess);
				
		return delete() || !users.isEmpty();
	}
	
}
